/**
* Eine Klasse zur Animation eines Folienübergangs bzw. dem Erscheinen eines Overlays
*/
function SlideTransition(
    name        // Name dieser Instanz
) {
    if (name)
        SlideTransition.all[name] = this;
}

// Alle bekannten SlideTransition-Instanzen
SlideTransition.all = {};

/**
* SlideTransition-Instanz anhand des Namens auswählen
* @return SlideTransition   Gesuchte SlideTransition-Instanz (oder null)
*/
SlideTransition.getByName = function(
    name    // Name der gewünschten Instanz
) {
    return SlideTransition.all[name] ? SlideTransition.all[name] : null;
}

/**
* Die logische Umkehr-Instanz finden
* @return SlideTransition   Umkehrung (oder this)
*/
SlideTransition.prototype.reverse = function() {
    return this;
}

/**
* Array mit Keyframes für neue Folie erzeugen
* @return Array     Keyframes (oder null)
*/
SlideTransition.prototype.getFramesNew = function(
    width,      // Folienbreite
    height      // Folienhöhe
) {
    return null;
}

/**
* Array mit Keyframes für alte Folie erzeugen
* @return Array     Keyframes (oder null)
*/
SlideTransition.prototype.getFramesOld = function(
    width,      // Folienbreite
    height      // Folienhöhe
) {
    return null;
}


/**
* Folie einschieben
*/
function SlideTransitionSlide(
    name,       // Name dieser Instanz
    horizontal, // Horizontale Bewegung (-1 => von links kommend, 0 => keine, +1 => von rechts kommend)
    vertical,   // Vertikale Bewegung (-1 => von unten kommend, 0 => keine, +1 => von oben kommend)
    moveOld     // Soll sich die alte Folie auch bewegen?
) {
    SlideTransitionSlide.parent.call(this, name);
    this.horizontal = horizontal;
    this.vertical = vertical;
    this.moveOld = moveOld;
}
SlideTransitionSlide.prototype = Object.create(SlideTransition.prototype);
SlideTransitionSlide.prototype.constructor = SlideTransitionSlide;
SlideTransitionSlide.parent = SlideTransition;

/**
* Einen Keyframe erstellen
* @return Object    Keyframe
*/
SlideTransitionSlide._createOneKeyframe = function(
    width,          // Folienbreite
    height,         // Folienhöhe
    horizontal,     // Horizontale Startposition (-1 => links, 0 => mittig, +1 => recht)
    vertical        // Vertikale Startposition (-1 => unten, 0 => mittig, +1 => oben)
) {
    return { transform: 'translate(' + (horizontal ? ((horizontal > 0) ? width : -width) : '0') + 'px, ' + (vertical ? ((vertical < 0) ? height : -height) : '0') + 'px)' };
}

/**
* Keyframes für gewünschte Animation bestimmen
* @return Array     Keyframes
*/
SlideTransitionSlide._createKeyframes = function(
    width,              // Folienbreite
    height,             // Folienhöhe
    startHorizontal,    // Horizontale Startposition (-1 => links, 0 => mittig, +1 => recht)
    startVertical,      // Vertikale Startposition (-1 => unten, 0 => mittig, +1 => oben)
    endHorizontal,      // Horizontale Endposition (-1 => links, 0 => mittig, +1 => recht)
    endVertical         // Vertikale Endposition (-1 => unten, 0 => mittig, +1 => oben)
) {
    return [
        SlideTransitionSlide._createOneKeyframe(width, height, startHorizontal, startVertical),
        SlideTransitionSlide._createOneKeyframe(width, height, endHorizontal, endVertical)
    ];
}

/**
* Die logische Umkehr-Instanz finden
* @return SlideTransition   Umkehrung (oder this)
*/
SlideTransitionSlide.prototype.reverse = function() {
    return (new SlideTransitionSlide(false, -this.horizontal, -this.vertical, this.moveOld));
}

/**
* Array mit Keyframes für neue Folie erzeugen
* @return Array     Keyframes (oder null)
*/
SlideTransitionSlide.prototype.getFramesNew = function(
    width,      // Folienbreite
    height      // Folienhöhe
) {
    return SlideTransitionSlide._createKeyframes(width, height, this.horizontal, this.vertical, 0, 0);
}

/**
* Array mit Keyframes für alte Folie erzeugen
* @return Array     Keyframes (oder null)
*/
SlideTransitionSlide.prototype.getFramesOld = function(
    width,      // Folienbreite
    height      // Folienhöhe
) {
    return this.moveOld ? SlideTransitionSlide._createKeyframes(width, height, 0, 0, -this.horizontal, -this.vertical) : null;
}




// Standardanimationen
var a = (new SlideTransition('gently'));
a.getFramesNew = function() { return [{opacity:0}, {opacity:1}]; };
a = new SlideTransition('spin');
a.getFramesNew = function(width, height) {
    var origin = (width / 2) + 'px ' + (height / 2) + 'px';
    return [
        { transform: 'rotate(0deg) scale(0)', 'transformOrigin': origin },
        { transform: 'rotate(3600deg) scale(1)', 'transformOrigin': origin }
    ];
}
new SlideTransitionSlide('slideover-fromleft', -1, 0, false);
new SlideTransitionSlide('slideover-fromright', +1, 0, false);
new SlideTransitionSlide('slideover-fromtop', 0, +1, false);
new SlideTransitionSlide('slideover-frombottom', 0, -1, false);
new SlideTransitionSlide('slideover-fromtopleft', -1, +1, false);
new SlideTransitionSlide('slideover-frombottomleft', -1, -1, false);
new SlideTransitionSlide('slideover-fromtopright', +1, +1, false);
new SlideTransitionSlide('slideover-frombottomright', +1, -1, false);
new SlideTransitionSlide('pushaway-fromleft', -1, 0, true);
new SlideTransitionSlide('pushaway-fromright', +1, 0, true);
new SlideTransitionSlide('pushaway-fromtop', 0, +1, true);
new SlideTransitionSlide('pushaway-frombottom', 0, -1, true);
new SlideTransitionSlide('pushaway-fromtopleft', -1, +1, true);
new SlideTransitionSlide('pushaway-frombottomleft', -1, -1, true);
new SlideTransitionSlide('pushaway-fromtopright', +1, +1, true);
new SlideTransitionSlide('pushaway-frombottomright', +1, -1, true);
